@@ -25,7 +25,6 @@
 #include <string.h>
 #include <glib-object.h>
 #include <glib.h>
-#include <glib/gstring.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -41,7 +40,6 @@
 #include <linux/videodev.h>
 
 #include "cheese-webcam.h"
-#include "cheese-flash.h"
 
 G_DEFINE_TYPE (CheeseWebcam, cheese_webcam, G_TYPE_OBJECT)
 
@@ -94,8 +92,6 @@ typedef struct
   int selected_device;
   CheeseVideoFormat *current_format;
 
-  CheeseFlash *flash;
-
   guint eos_timeout_id;
 } CheeseWebcamPrivate;
 
@@ -219,6 +215,7 @@ cheese_webcam_photo_data_cb (GstElement *element, GstBuffer *buffer,
 
   g_signal_handler_disconnect (G_OBJECT (priv->photo_sink),
                                priv->photo_handler_signal_id);
+  priv->photo_handler_signal_id = 0;
 
   g_signal_emit (webcam, webcam_signals[PHOTO_SAVED], 0);
 }
@@ -257,18 +254,20 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
 
   priv->num_webcam_devices = 0;
 
+  g_print ("Probing devices with HAL...\n");
+
   dbus_error_init (&error);
   hal_ctx = libhal_ctx_new ();
   if (hal_ctx == NULL)
   {
-    g_error ("error: libhal_ctx_new");
+    g_warning ("Could not create libhal context");
     dbus_error_free (&error);
     goto fallback;
   }
 
   if (!libhal_ctx_set_dbus_connection (hal_ctx, dbus_bus_get (DBUS_BUS_SYSTEM, &error)))
   {
-    g_error ("error: libhal_ctx_set_dbus_connection: %s: %s", error.name, error.message);
+    g_warning ("libhal_ctx_set_dbus_connection: %s: %s", error.name, error.message);
     dbus_error_free (&error);
     goto fallback;
   }
@@ -277,11 +276,11 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
   {
     if (dbus_error_is_set (&error))
     {
-      g_error ("error: libhal_ctx_init: %s: %s\n", error.name, error.message);
+      g_warning ("libhal_ctx_init: %s: %s", error.name, error.message);
       dbus_error_free (&error);
     }
-    g_error ("Could not initialise connection to hald.\n"
-             "Normally this means the HAL daemon (hald) is not running or not ready");
+    g_warning ("Could not initialise connection to hald.\n"
+               "Normally this means the HAL daemon (hald) is not running or not ready");
     goto fallback;
   }
 
@@ -289,7 +288,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
 
   if (dbus_error_is_set (&error))
   {
-    g_error ("error: libhal_find_device_by_capability: %s: %s\n", error.name, error.message);
+    g_warning ("libhal_find_device_by_capability: %s: %s", error.name, error.message);
     dbus_error_free (&error);
     goto fallback;
   }
@@ -300,14 +299,50 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
   for (i = 0; i < num_udis; i++)
   {
     char                   *device;
+    char                   *parent_udi = NULL;
+    char                   *subsystem = NULL;
     char                   *gstreamer_src, *product_name;
     struct v4l2_capability  v2cap;
     struct video_capability v1cap;
+    gint vendor_id = 0;
+    gint product_id = 0;
+    gchar *property_name = NULL;
+
+    parent_udi = libhal_device_get_property_string (hal_ctx, udis[i], "info.parent", &error);
+    if (dbus_error_is_set (&error))
+    {
+      g_warning ("error getting parent for %s: %s: %s", udis[i], error.name, error.message);
+      dbus_error_free (&error);
+    }
+
+    if (parent_udi != NULL) {
+      subsystem = libhal_device_get_property_string (hal_ctx, parent_udi, "info.subsystem", NULL);
+      if (subsystem == NULL) continue;
+      property_name = g_strjoin (".", subsystem, "vendor_id", NULL);
+      vendor_id = libhal_device_get_property_int (hal_ctx, parent_udi, property_name , &error);
+      if (dbus_error_is_set (&error)) {
+        g_warning ("error getting vendor id: %s: %s", error.name, error.message);
+        dbus_error_free (&error);
+      }
+      g_free (property_name);
+
+      property_name = g_strjoin (".", subsystem, "product_id", NULL);
+      product_id = libhal_device_get_property_int (hal_ctx, parent_udi, property_name, &error);
+      if (dbus_error_is_set (&error)) {
+        g_warning ("error getting product id: %s: %s", error.name, error.message);
+        dbus_error_free (&error);
+      }
+      g_free (property_name);
+      libhal_free_string (subsystem);
+      libhal_free_string (parent_udi);
+    }
+
+    g_print ("Found device %04x:%04x, getting capabilities...\n", vendor_id, product_id);
 
     device = libhal_device_get_property_string (hal_ctx, udis[i], "video4linux.device", &error);
     if (dbus_error_is_set (&error))
     {
-      g_error ("error geting device for %s: %s: %s\n", udis[i], error.name, error.message);
+      g_warning ("error getting V4L device for %s: %s: %s", udis[i], error.name, error.message);
       dbus_error_free (&error);
       continue;
     }
@@ -323,7 +358,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
 
     if ((fd = open (device, O_RDONLY | O_NONBLOCK)) < 0)
     {
-      g_error ("Failed to open %s: %s\n", device, strerror (errno));
+      g_warning ("Failed to open %s: %s", device, strerror (errno));
       libhal_free_string (device);
       continue;
     }
@@ -333,7 +368,7 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
       ok = ioctl (fd, VIDIOCGCAP, &v1cap);
       if (ok < 0)
       {
-        g_error ("Error while probing v4l capabilities for %s: %s\n",
+        g_warning ("Error while probing v4l capabilities for %s: %s",
                  device, strerror (errno));
         libhal_free_string (device);
         close (fd);
@@ -349,8 +384,8 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
       guint cap = v2cap.capabilities;
       g_print ("Detected v4l2 device: %s\n", v2cap.card);
       g_print ("Driver: %s, version: %d\n", v2cap.driver, v2cap.version);
-      g_print ("Bus info: %s\n", v2cap.bus_info);
-      g_print ("Capabilities: 0x%08X\n", v2cap.capabilities);
+      /* g_print ("Bus info: %s\n", v2cap.bus_info); */ /* Doesn't seem anything useful */
+     g_print ("Capabilities: 0x%08X\n", v2cap.capabilities);
       if (!(cap & V4L2_CAP_VIDEO_CAPTURE))
       {
         g_print ("Device %s seems to not have the capture capability, (radio tuner?)\n"
@@ -363,6 +398,8 @@ cheese_webcam_get_video_devices_from_hal (CheeseWebcam *webcam)
       product_name  = (char *) v2cap.card;
     }
 
+    g_print ("\n");
+
     priv->webcam_devices[priv->num_webcam_devices].hal_udi           = g_strdup (udis[i]);
     priv->webcam_devices[priv->num_webcam_devices].video_device      = g_strdup (device);
     priv->webcam_devices[priv->num_webcam_devices].gstreamer_src     = g_strdup (gstreamer_src);
@@ -562,7 +599,9 @@ cheese_webcam_get_supported_video_formats (CheeseWebcamDevice *webcam_device, Gs
 
       cur_width  = min_width;
       cur_height = min_height;
-      while (cur_width < max_width && cur_height < max_height)
+      /* Gstreamer will sometimes give us a range with min_xxx == max_xxx,
+         we use <= here (and not below) to make this work */
+      while (cur_width <= max_width && cur_height <= max_height)
       {
         CheeseVideoFormat video_format;
 
@@ -652,8 +691,7 @@ cheese_webcam_get_webcam_device_data (CheeseWebcam       *webcam,
         if (name == NULL)
           name = "Unknown";
 
-        g_print ("Detected webcam: %s\n", name);
-        g_print ("device: %s\n", webcam_device->video_device);
+        g_print ("Device: %s (%s)\n", name, webcam_device->video_device);
         pad  = gst_element_get_pad (src, "src");
         caps = gst_pad_get_caps (pad);
         gst_object_unref (pad);
@@ -699,9 +737,12 @@ cheese_webcam_detect_webcam_devices (CheeseWebcam *webcam)
   int i;
 
   cheese_webcam_get_video_devices_from_hal (webcam);
+
+  g_print ("Probing supported video formats...\n");
   for (i = 0; i < priv->num_webcam_devices; i++)
   {
     cheese_webcam_get_webcam_device_data (webcam, &(priv->webcam_devices[i]));
+    g_print ("\n");
   }
 
   if (priv->num_webcam_devices == 0)
@@ -1303,11 +1344,16 @@ cheese_webcam_stop_video_recording (CheeseWebcam *webcam)
   }
 }
 
-void
+gboolean
 cheese_webcam_take_photo (CheeseWebcam *webcam, char *filename)
 {
   CheeseWebcamPrivate *priv = CHEESE_WEBCAM_GET_PRIVATE (webcam);
 
+  if (priv->photo_handler_signal_id != 0) {
+    g_print ("Still waiting for previous photo data, ignoring new request\n");
+    return FALSE;
+  }
+
   g_free (priv->photo_filename);
   priv->photo_filename = g_strdup (filename);
 
@@ -1316,7 +1362,7 @@ cheese_webcam_take_photo (CheeseWebcam *webcam, char *filename)
                                                     "handoff",
                                                     G_CALLBACK (cheese_webcam_photo_data_cb),
                                                     webcam);
-  cheese_flash_fire (priv->flash);
+  return TRUE;
 }
 
 static void
@@ -1491,8 +1537,7 @@ cheese_webcam_init (CheeseWebcam *webcam)
   priv->photo_filename      = NULL;
   priv->webcam_devices      = NULL;
   priv->device_name         = NULL;
-
-  priv->flash = cheese_flash_new ();
+  priv->photo_handler_signal_id = 0;
 }
 
 CheeseWebcam *
