@@ -55,6 +55,7 @@
 #include "gedit-message-area.h"
 #include "cheese-no-camera.h"
 #include "cheese-prefs-dialog.h"
+#include "cheese-flash.h"
 
 #define SHUTTER_SOUNDS             5
 #define FULLSCREEN_POPUP_HEIGHT    40
@@ -87,7 +88,6 @@ typedef struct
   /* UDI device requested on the command line */
   char *startup_hal_dev_udi;
   char *video_filename;
-  char *photo_filename;
 
   int counter;
 
@@ -172,6 +172,8 @@ typedef struct
   GSource *fullscreen_timeout_source;
 
   int audio_play_counter;
+
+  CheeseFlash *flash;
 } CheeseWindow;
 
 void
@@ -191,10 +193,11 @@ cheese_about_dialog_handle_url (GtkAboutDialog *dialog, const char *url, gpointe
 {
   GError    *error = NULL;
   GtkWidget *error_dialog;
-  gboolean   ret;
+  GdkScreen *screen;
 
-  ret = g_app_info_launch_default_for_uri (url, NULL, &error);
-  if (ret == FALSE)
+  screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
+  gtk_show_uri (screen, url, gtk_get_current_event_time (), &error);
+  if (error != NULL)
   {
     error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
                                            GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -213,12 +216,13 @@ cheese_about_dialog_handle_email (GtkAboutDialog *dialog, const char *email, gpo
   char      *uri;
   GError    *error = NULL;
   GtkWidget *error_dialog;
-  gboolean   ret;
+  GdkScreen *screen;
 
   uri = g_strconcat ("mailto:", email, NULL);
 
-  ret = g_app_info_launch_default_for_uri (uri, NULL, &error);
-  if (ret == FALSE)
+  screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
+  gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+  if (error != NULL)
   {
     error_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog),
                                            GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -402,22 +406,26 @@ cheese_window_fullscreen_leave_notify_cb (GtkWidget        *widget,
 static void
 cheese_window_photo_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
-  /* TODO look at this g_free */
-  g_free (cheese_window->photo_filename);
-  cheese_window->photo_filename = NULL;
+  gdk_threads_enter ();
   gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
   gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, TRUE);
+  gdk_flush ();
+  gdk_threads_leave ();
 }
 
+
 static void
 cheese_window_video_saved_cb (CheeseWebcam *webcam, CheeseWindow *cheese_window)
 {
+  gdk_threads_enter ();
   /* TODO look at this g_free */
   g_free (cheese_window->video_filename);
   cheese_window->video_filename = NULL;
   gtk_action_group_set_sensitive (cheese_window->actions_effects, TRUE);
   gtk_widget_set_sensitive (cheese_window->take_picture, TRUE);
   gtk_widget_set_sensitive (cheese_window->take_picture_fullscreen, TRUE);
+  gdk_flush ();
+  gdk_threads_leave ();
 }
 
 static void
@@ -451,17 +459,19 @@ cheese_window_cmd_open (GtkWidget *widget, CheeseWindow *cheese_window)
 {
   char      *uri;
   char      *filename;
-  gboolean   ret;
   GError    *error = NULL;
   GtkWidget *dialog;
+  GdkScreen *screen;
 
   filename = cheese_thumb_view_get_selected_image (CHEESE_THUMB_VIEW (cheese_window->thumb_view));
   g_return_if_fail (filename);
   uri = g_filename_to_uri (filename, NULL, NULL);
   g_free (filename);
 
-  ret = g_app_info_launch_default_for_uri (uri, NULL, &error);
-  if (ret == FALSE)
+  screen = gtk_widget_get_screen (GTK_WIDGET (cheese_window->window));
+  gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+
+  if (error != NULL)
   {
     dialog = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
                                      GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -866,7 +876,7 @@ cheese_window_cmd_command_line (GtkAction *action, CheeseWindow *cheese_window)
   if (strcmp (action_name, "SendByMail") == 0)
   {
     char *path;
-    command_line = g_strdup_printf ("gnome-open mailto:?subject='%s'", _("Media files"));
+    command_line = g_strdup_printf ("xdg-open mailto:?subject='%s'", _("Media files"));
     for (l = files; l != NULL; l = l->next)
     {
       path         = g_file_get_path (l->data);
@@ -929,11 +939,12 @@ static void
 cheese_window_cmd_help_contents (GtkAction *action, CheeseWindow *cheese_window)
 {
   GError  *error = NULL;
-  gboolean ret;
+  GdkScreen *screen;
 
-  ret = g_app_info_launch_default_for_uri ("ghelp:cheese", NULL, &error);
+  screen = gtk_widget_get_screen (GTK_WIDGET (cheese_window));
+  gtk_show_uri (screen, "ghelp:cheese", gtk_get_current_event_time (), &error);
 
-  if (ret == FALSE)
+  if (error != NULL)
   {
     GtkWidget *d;
     d = gtk_message_dialog_new (GTK_WINDOW (cheese_window->window),
@@ -968,7 +979,7 @@ cheese_window_cmd_about (GtkAction *action, CheeseWindow *cheese_window)
     "Ryan Zeigler <zeiglerr@gmail.com>",
     "Sebastian Keller <sebastian-keller@gmx.de>",
     "Steve Magoun <steve.magoun@canonical.com>",
-    "Thomas Perl <thp@perli.ne>",
+    "Thomas Perl <thp@thpinfo.com>",
     "Tim Philipp Müller <tim@centricular.net>",
     "Todd Eisenberger <teisenberger@gmail.com>",
     "Tommi Vainikainen <thv@iki.fi>",
@@ -1018,7 +1029,7 @@ cheese_window_cmd_about (GtkAction *action, CheeseWindow *cheese_window)
                          "translator-credits", translators,
                          "artists", artists,
                          "documenters", documenters,
-                         "website", "http://www.gnome.org/projects/cheese",
+                         "website", "http://projects.gnome.org/cheese",
                          "website-label", _("Cheese Website"),
                          "logo-icon-name", "cheese",
                          "wrap-license", TRUE,
@@ -1179,20 +1190,23 @@ cheese_window_countdown_picture_cb (gpointer data)
   CheeseWindow *cheese_window = (CheeseWindow *) data;
   GError       *error         = NULL;
   GstAudioPlay *audio_play;
-  char         *file;
+  char         *shutter_filename;
+  char         *photo_filename;
 
-  file       = audio_play_get_filename (cheese_window);
-  audio_play = gst_audio_play_file (file, &error);
-  if (!audio_play)
+  photo_filename = cheese_fileutil_get_new_media_filename (cheese_window->fileutil, WEBCAM_MODE_PHOTO);
+  if (cheese_webcam_take_photo (cheese_window->webcam, photo_filename))
   {
-    g_warning ("%s", error ? error->message : "Unknown error");
-    g_error_free (error);
+    cheese_flash_fire (cheese_window->flash);
+    shutter_filename = audio_play_get_filename (cheese_window);
+    audio_play = gst_audio_play_file (shutter_filename, &error);
+    if (!audio_play)
+    {
+      g_warning ("%s", error ? error->message : "Unknown error");
+      g_error_free (error);
+    }
+    g_free (shutter_filename);
   }
-
-  g_free (file);
-
-  cheese_window->photo_filename = cheese_fileutil_get_new_media_filename (cheese_window->fileutil, WEBCAM_MODE_PHOTO);
-  cheese_webcam_take_photo (cheese_window->webcam, cheese_window->photo_filename);
+  g_free (photo_filename);
 }
 
 static void
@@ -1617,8 +1631,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   cheese_window->main_hbox = gtk_hbox_new (FALSE, 0);
   hildon_program_add_window (program, HILDON_WINDOW (cheese_window->window));
   gtk_container_add (GTK_CONTAINER (cheese_window->window), cheese_window->main_hbox);
-  gtk_widget_ref (cheese_window->thumb_scrollwindow);
-  gtk_widget_ref (cheese_window->video_vbox);
+  g_object_ref (cheese_window->thumb_scrollwindow);
+  g_object_ref (cheese_window->video_vbox);
   gtk_container_remove (GTK_CONTAINER (cheese_window->video_vbox), cheese_window->thumb_scrollwindow);
   gtk_container_remove (GTK_CONTAINER (cheese_window->main_vbox), cheese_window->video_vbox);
   gtk_box_pack_start (GTK_BOX (cheese_window->main_hbox), cheese_window->video_vbox, TRUE, TRUE, 0);
@@ -1627,8 +1641,8 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   gtk_widget_destroy (cheese_window->main_vbox);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (cheese_window->thumb_scrollwindow),
                                   GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_widget_unref (cheese_window->thumb_scrollwindow);
-  gtk_widget_unref (cheese_window->video_vbox);
+  g_object_unref (cheese_window->thumb_scrollwindow);
+  g_object_unref (cheese_window->video_vbox);
 #endif
 
   g_object_unref (builder);
@@ -1730,7 +1744,7 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   }
   else
   {
-    path = g_find_program_in_path ("gnome-open");
+    path = g_find_program_in_path ("xdg-open");
     gtk_action_group_set_visible (cheese_window->actions_mail, path != NULL);
     gtk_action_group_set_visible (cheese_window->actions_sendto, FALSE);
   }
@@ -1778,15 +1792,15 @@ cheese_window_create_window (CheeseWindow *cheese_window)
   menu     = gtk_menu_new ();
   menuitem = gtk_menu_item_new_with_label (_("Quit"));
   g_signal_connect (menuitem, "activate",
-                    GTK_SIGNAL_FUNC (cheese_window_cmd_close),
+                    G_CALLBACK (cheese_window_cmd_close),
                     cheese_window);
-  gtk_menu_append (menu, menuitem);
+  gtk_menu_shell_append (menu, menuitem);
 
   menuitem = gtk_menu_item_new_with_label (_("About"));
   g_signal_connect (menuitem, "activate",
-                    GTK_SIGNAL_FUNC (cheese_window_cmd_about),
+                    G_CALLBACK (cheese_window_cmd_about),
                     cheese_window);
-  gtk_menu_append (menu, menuitem);
+  gtk_menu_shell_append (menu, menuitem);
 
   hildon_window_set_menu (HILDON_WINDOW (cheese_window->window), GTK_MENU (menu));
 #else
@@ -1874,7 +1888,7 @@ setup_camera (CheeseWindow *cheese_window)
                                      GTK_BUTTONS_OK,
                                      "%s", primary);
     gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
-                                                secondary);
+                                                "%s", secondary);
     gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
     g_error_free (error);
@@ -1927,8 +1941,9 @@ cheese_window_init (char *hal_dev_udi, CheeseDbus *dbus_server)
 
   cheese_window->startup_hal_dev_udi = hal_dev_udi;
   cheese_window->gconf               = cheese_gconf_new ();
-  cheese_window->audio_play_counter  = 0;
   cheese_window->fileutil            = cheese_fileutil_new ();
+  cheese_window->flash               = cheese_flash_new ();
+  cheese_window->audio_play_counter  = 0;
   cheese_window->isFullscreen        = FALSE;
 
   cheese_window->server = dbus_server;
